function [thresh, metric] = multithresh(varargin)
  %{ 
 -- Function File:  thresh = multithresh(I)
 -- Function File: [thresh, metric] = multithresh(I)
 -- Function File:  thresh = multithresh(I,N)
 -- Function File: [thresh, metric] = multithresh(I,N)

     Given an image I finds the optimal threshold values,
     The first output "thresh", using Otsu's method as 
     described in 'Nobuyuki Otsu(1979).
     "A threshold selection method from gray-level
     histograms", IEEE Trans. Sys., Man., Cyber. 9 (1): 62-66'.
     This algorithm chooses the threshold to minimize the
     intraclass variance of the black and white pixels.

     The second output, metric represents the "goodness" (or
     separability) of the threshold at thresh.  It is a value within
     the range [0 1].
      
     N is the second argument is the number of thresholds needed to segment 
     the input image. It has a default value of 1.
          %}
        
   if (nargin == 0 || nargin>2)
       error (usage());
   endif
   % the input image 
   I=varargin{1};
   if (!isgray(I) && !isrgb(I))
      error("multithresh: The input must be an image\n");
    endif
   % default value of N
   N=1; 
 
   if (nargin==2)
     N=varargin{2};
     if (!isnumeric (N))
       error("multithresh: N must be integral, single and postive\n");
     elseif size(N)>1 || N-floor(N)!=0 || N<0
        error("multithresh: N must be integral, single and postive\n");
     end
   end
   
   metric=0;
   % calculate the histogram
   [counts,x]=imhist(I);
   if(N==1)
     [metric,level]=Onethreshold(counts);
     if(level==-1)
       thresh=0;
     else
        thresh=x(level);
        
     end
     return
   end
   %  Otsu's method
    p=counts/sum(counts);
    w=cumsum(p);
    u=cumsum([1:length(counts)]'.*p);
    uT=u(end);
    s_t2=(sum(p.*(((1:length(counts))' - uT).^2)));
   % initial value of the threshold
   pre_thresh=floor([1:N]*length(counts)/(N+1));
   pre_thresh0=pre_thresh;
   max_s_b2=-inf;
   for i=1:length(counts)
     s_b2=0;
       for j=1:N
          areaboundaries=[1 pre_thresh length(counts)];
          [~,level]=Onethreshold(counts(areaboundaries(j):areaboundaries(j+2)));
          if(level>0)
            pre_thresh(j)=level+areaboundaries(j);
          end
        end
        wv=w(pre_thresh(1));
        if(wv==0)
          wv=1;
        end
        uv=u(pre_thresh(1));
        uv=uv/wv;
        s_b2=s_b2+(uv-uT)^2*wv;
      for k=2:N+1
         wv=w(areaboundaries(k+1))-w(areaboundaries(k));
         uv=u(areaboundaries(k+1))-u(areaboundaries(k));
        if(wv==0)
          continue;
        end
         uv=uv/wv;
        s_b2=s_b2+(uv-uT)^2*wv;
      end
      if(max_s_b2<s_b2)
        max_s_b2=s_b2;
        thresh=x(pre_thresh)';
        if s_t2>0
        metric = max_s_b2/s_t2;
         end
    end
    % check if the thresholds coverage
  if(sum((pre_thresh-pre_thresh0).^2)==0)
    return
  end
      pre_thresh0=pre_thresh;
  end
  
endfunction
%------------------------------------------------------------------------
function msg=usage ()
  line1="multithresh: Invalid call to multithresh.  Correct usage is\n";
  line2="  -- Function File:  thresh = multithresh(I)\n ";
  line3=" -- Function File: [thresh, metric] = multithresh(I)\n ";
  line4=" -- Function File: thresh = multithresh(I,N)\n ";
  line5=" -- Function File: [thresh, metric] = multithresh(I,N)\n ";
  msg=[line1,line2,line3,line4,line5,"\n"];
endfunction
%------------------------------------------------------------------------
function [metric ,level]=Onethreshold(subcounts)

  level=-1;
  metric=0;
  H=sum(subcounts);
  if(H==0)
   return;
  end

  % Calculation of the cumulated histogram and the mean values
  p=subcounts/H;
  w=cumsum(p);
  u=cumsum([1:length(subcounts)]'.*p);
  uT=u(end);
  s_t2=(sum(p.*(((1:length(subcounts))' - uT).^2)));
  s_b2=[uT*w-u].^2./(w.*(1-w));

  [max_s_b2,level]=max(s_b2);
  if s_t2==0
   return
  end
  metric = max_s_b2/s_t2;
endfunction

